package com.ningdatech.pmapi.common.util;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.DefaultHostnameVerifier;
import org.apache.http.conn.util.PublicSuffixMatcher;
import org.apache.http.conn.util.PublicSuffixMatcherLoader;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
public class HttpUtil {
    private Logger logger = LoggerFactory.getLogger(getClass());
    private RequestConfig requestConfig = RequestConfig.custom()
        .setSocketTimeout(15000)
        .setConnectTimeout(15000)
        .setConnectionRequestTimeout(15000)
        .build();
    private static HttpUtil instance = null;
    private HttpUtil(){}

    public static HttpUtil getInstance(){
        if (instance == null) {
            instance = new HttpUtil();
        }
        return instance;
    }

    /**
    * 发送 post 请求
    * @param httpUrl 地址
    */
    public String sendHttpPost(String httpUrl) {
        HttpPost httpPost= new HttpPost(httpUrl);// 创建 httpPost
        return sendHttpPost(httpPost);
    }

    /**
    * 发送 post 请求
    * @param httpUrl 地址
    * @param params 参数(格式:key1=value1&key2=value2)
    */
    public String sendHttpPost(String httpUrl, String params) {
        HttpPost httpPost= new HttpPost(httpUrl);// 创建 httpPost
        try {
            //设置参数
            StringEntity stringEntity = new StringEntity(params, "UTF-8");
            stringEntity.setContentType("application/x-www-form-urlencoded");
            httpPost.setEntity(stringEntity);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        }
        return sendHttpPost(httpPost);
    }

    /**
    * 发送 post 请求
    * @param httpUrl 地址
    * @param maps 参数
    */
    public String sendHttpPost(String httpUrl, Map<String, String> maps) {
        HttpPost httpPost= new HttpPost(httpUrl);// 创建 httpPost
        // 创建参数队列
        List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
        for (String key : maps.keySet()) {
            nameValuePairs.add(new BasicNameValuePair(key, maps.get(key)));
        }
        try {
            httpPost.setEntity(new UrlEncodedFormEntity(nameValuePairs, "UTF-8"));
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        }
        return sendHttpPost(httpPost);
    }

    /**
    * 发送 Post请求
    * @param httpPost
    *@return
    */
    private String sendHttpPost(HttpPost httpPost) {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse response = null;
        HttpEntity entity = null;
        String responseContent = null;
        try {
            // 创建默认的 httpClient 实例.
            httpClient = HttpClients.createDefault();
            httpPost.setConfig(requestConfig);
            // 执行请求
            response = httpClient.execute(httpPost);
            entity = response.getEntity();
            responseContent = EntityUtils.toString(entity, "UTF-8");
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        } finally {
            try {
                // 关闭连接,释放资源
                if (response != null) {
                response.close();
                }
                if (httpClient != null) {
                httpClient.close();
                }
            } catch (IOException e) {
                logger.error(e.getMessage(),e);
            }
        }
        return responseContent;
    }

    /**
    * 发送 get 请求
    * @param httpUrl
    */
    public String sendHttpGet(String httpUrl) {
        HttpGet httpGet = new HttpGet(httpUrl);// 创建 get 请求
        return sendHttpGet(httpGet);
    }

    /**
    * 发送 get请求 Https
    * @param httpUrl
    */
    public String sendHttpsGet(String httpUrl) {
        HttpGet httpGet = new HttpGet(httpUrl);// 创建 get 请求
        return sendHttpsGet(httpGet);
    }

    /**
    * 发送 Get请求
    * @param httpGet
    *@return
    */
    private String sendHttpGet(HttpGet httpGet) {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse response = null;
        HttpEntity entity = null;
        String responseContent = null;
        try {
            // 创建默认的 httpClient 实例.
            httpClient = HttpClients.createDefault();
            httpGet.setConfig(requestConfig);
            // 执行请求
            response = httpClient.execute(httpGet);
            entity = response.getEntity();
            responseContent = EntityUtils.toString(entity, "UTF-8");
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        } finally {
            try {
                // 关闭连接,释放资源
                if (response != null) {
                    response.close();
                }
                if (httpClient != null) {
                    httpClient.close();
                }
            } catch (IOException e) {
                logger.error(e.getMessage(),e);
            }
        }
        return responseContent;
    }

    /**
    * 发送 Get请求 Https
    *@return
    */
    private String sendHttpsGet(HttpGet httpGet) {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse response = null;
        HttpEntity entity = null;
        String responseContent = null;
        try {
            // 创建默认的 httpClient 实例.
            PublicSuffixMatcher publicSuffixMatcher = PublicSuffixMatcherLoader.load(new
            URL(httpGet.getURI().toString()));
            DefaultHostnameVerifier hostnameVerifier = new DefaultHostnameVerifier(publicSuffixMatcher);
            httpClient = HttpClients.custom().setSSLHostnameVerifier(hostnameVerifier).build();
            httpGet.setConfig(requestConfig);
            // 执行请求
            response = httpClient.execute(httpGet);
            entity = response.getEntity();
            responseContent = EntityUtils.toString(entity, "UTF-8");
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        } finally {
            try {
                // 关闭连接,释放资源
                if (response != null) {
                    response.close();
                }
                if (httpClient != null) {
                    httpClient.close();
                }
            } catch (IOException e) {
                logger.error(e.getMessage(),e);
            }
        }
        return responseContent;
    }
}