package com.ningdatech.pmapi.ding.controller;

import com.alibaba.fastjson.JSON;
import com.ningdatech.pmapi.common.util.CryptUtils;
import com.ningdatech.pmapi.common.util.RefreshKeyUtil;
import com.ningdatech.pmapi.ding.task.EmployeeBatchGetTask;
import com.ningdatech.pmapi.ding.task.GovBusinessStripsTask;
import com.ningdatech.pmapi.ding.task.OrganizationBatchGetTask;
import com.ningdatech.pmapi.organization.manage.ProcDefManage;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.security.NoSuchAlgorithmException;

/**
 * <p>
 * PullDingInfoController
 * </p>
 *
 * @author WendyYang
 * @since 2023/6/2
 **/
@RestController
@RequestMapping("/api/v1/zwdd/pull")
@RequiredArgsConstructor
@Slf4j
public class DingInfoPullController {

    private final EmployeeBatchGetTask employeeBatchGetTask;
    private final GovBusinessStripsTask businessStripsTask;
    private final OrganizationBatchGetTask organizationBatchGetTask;

    private final ProcDefManage procDefManage;


    @GetMapping("/employee")
    public void employeeBatchGet() {
        employeeBatchGetTask.batchGetEmployeeTask();
    }

    @GetMapping("/businessStrips")
    public void businessStripsBatchGet() {
        businessStripsTask.batchGetGovBusinessStripsTask();
    }

    @GetMapping("/organization")
    public void organizationBatchGet() {
        organizationBatchGetTask.batchGetOrganizationTask();
    }

    @GetMapping("/employeeByRegionCode/{regionCode}")
    public void employeeBatchGetByRegionCode(@PathVariable String regionCode) {
        employeeBatchGetTask.batchGetEmployeeTaskByRegionCode(regionCode);
    }

    @GetMapping("/employeeByOrdCode/{orgCode}")
    public void employeeBatchGetByOrdCode(@PathVariable String orgCode) {
        employeeBatchGetTask.batchGetEmployeeTaskByOrdCode(orgCode);
    }

    @GetMapping("/getEmployeeByCode/{empCode}")
    public void getEmployeeByCode(@PathVariable String empCode) {
        employeeBatchGetTask.getEmployeeByCode(empCode);
    }

    @GetMapping("/getBatchEmployeeByCode/{orgCode}")
    public void getBatchEmployeeByCode(@PathVariable String orgCode) {
        employeeBatchGetTask.getBatchEmployeeByCode(orgCode);
    }

    @GetMapping("/getToken")
    public String getToken() {
       return employeeBatchGetTask.getToken();
    }

    @GetMapping("/getAuthToken")
    public String getAuthToken() {
        return employeeBatchGetTask.getAuthToken();
    }

    @ApiOperation(value = "系统流程配置初始化", notes = "系统流程配置初始化")
    @GetMapping("/init")
    public String init() {
        return procDefManage.init();
    }
}
