package com.ningdatech.pmapi.performance.controller;

import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import com.ningdatech.pmapi.performance.manage.IndicatorTemplateManage;
import com.ningdatech.pmapi.performance.model.dto.PerformanceIndicatorProjectTemplateSaveDTO;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.model.vo.PerformanceIndicatorProjectTemplateVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * @Classname PerformanceIndicatorTemplateController
 * @Description
 * @Date 2023/6/25 14:16
 * @Author PoffyZhang
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "绩效评价-指标配置")
@RequestMapping("/api/v1/indicator-template")
public class PerformanceIndicatorTemplateController {

    private final IndicatorTemplateManage indicatorTemplateManage;

    @GetMapping("/project/list")
    @ApiOperation("项目指标配置模板列表")
    public PageVo<PerformanceIndicatorProjectTemplateVO> projectList(PerformanceAppraisalListReq req) {
        return indicatorTemplateManage.projectList(req);
    }

    @GetMapping("/project/detail/{id}")
    @ApiOperation("项目指标配置模板详情")
    public PerformanceIndicatorProjectTemplateVO projectDetail(@PathVariable Long id) {
        return indicatorTemplateManage.projectDetail(id);
    }

    @PostMapping("/project/save")
    @ApiOperation("项目指标配置模板保存")
    @WebLog("项目指标配置模板保存")
    public String projectTemplateSave(@Valid @RequestBody PerformanceIndicatorProjectTemplateSaveDTO dto) {
        return indicatorTemplateManage.projectTemplateSave(dto);
    }

    @PostMapping("/project/delete/{id}")
    @ApiOperation("项目指标配置模板删除")
    @WebLog("项目指标配置模板删除")
    public String projectTemplateDelete(@PathVariable Long id) {
        return indicatorTemplateManage.projectTemplateDelete(id);
    }
}
