package com.ningdatech.pmapi.performance.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.enumeration.CommonEnum;
import com.ningdatech.pmapi.performance.enumration.PerformanceTemplateStageNameEnum;
import com.ningdatech.pmapi.performance.enumration.PerformanceTemplateTypeEnum;
import com.ningdatech.pmapi.performance.model.dto.PerformanceIndicatorProjectTemplateSaveDTO;
import com.ningdatech.pmapi.performance.model.dto.ProjectTemplateDetailDTO;
import com.ningdatech.pmapi.performance.model.entity.PerformanceIndicatorProjectTemplate;
import com.ningdatech.pmapi.performance.model.entity.PerformanceIndicatorProjectTemplateDetail;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.model.vo.PerformanceIndicatorProjectTemplateVO;
import com.ningdatech.pmapi.performance.model.vo.ProjectTemplateDetailVO;
import com.ningdatech.pmapi.performance.service.IPerformanceIndicatorProjectTemplateDetailService;
import com.ningdatech.pmapi.performance.service.IPerformanceIndicatorProjectTemplateService;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname IndicatorTemplateManage
 * @Description
 * @Date 2023/6/26 14:15
 * @Author PoffyZhang
 */
@Slf4j
@Component
@AllArgsConstructor
public class IndicatorTemplateManage {
    private final IPerformanceIndicatorProjectTemplateService indicatorProjectTemplateService;

    private final IPerformanceIndicatorProjectTemplateDetailService indicatorProjectTemplateDetailService;

    /**
     * 模板列表
     * @param req
     * @return
     */
    public PageVo<PerformanceIndicatorProjectTemplateVO> projectList(PerformanceAppraisalListReq req) {
        Page<PerformanceIndicatorProjectTemplate> page = req.page();
        LambdaQueryWrapper<PerformanceIndicatorProjectTemplate> wrapper = Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplate.class)
                .like(StringUtils.isNotBlank(req.getTemplateName()), PerformanceIndicatorProjectTemplate::getName, req.getTemplateName())
                .eq(StringUtils.isNotBlank(req.getRegionCode()), PerformanceIndicatorProjectTemplate::getRegionCode,req.getRegionCode());
        indicatorProjectTemplateService.page(page,wrapper);

        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<PerformanceIndicatorProjectTemplateVO> res = page.getRecords().stream()
                .map(p -> BeanUtil.copyProperties(p, PerformanceIndicatorProjectTemplateVO.class))
                .collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    /**
     * 查询项目模板详情
     * @param id
     * @return
     */
    public PerformanceIndicatorProjectTemplateVO projectDetail(Long id) {
        PerformanceIndicatorProjectTemplate template = indicatorProjectTemplateService.getById(id);
        if(Objects.isNull(template)){
            return null;
        }
        PerformanceIndicatorProjectTemplateVO vo = BeanUtil.copyProperties(template,
                PerformanceIndicatorProjectTemplateVO.class);
        List<PerformanceIndicatorProjectTemplateDetail> templateDetails = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, id));
        if(CollUtil.isNotEmpty(templateDetails)){
            vo.setTemplateDetails(buildTemplateDetail(templateDetails));
        }
        return vo;
    }

    /**
     * 项目指标 模板保存
     * @param dto
     * @return
     */
    @Transactional
    public String projectTemplateSave(PerformanceIndicatorProjectTemplateSaveDTO dto) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        PerformanceIndicatorProjectTemplate template = BeanUtil.copyProperties(dto, PerformanceIndicatorProjectTemplate.class);
        if(Objects.nonNull(dto.getId())){
            PerformanceIndicatorProjectTemplate old = indicatorProjectTemplateService.getById(dto.getId());
            VUtils.isTrue(Objects.isNull(old)).throwMessage("该模板不存在!");
            template.setId(old.getId());
        }else{
            template.setId(null);
            template.setCreateOn(LocalDateTime.now());
            template.setCreateBy(user.getUsername());
        }
        template.setUpdateOn(LocalDateTime.now());
        template.setUpdateBy(user.getUsername());
        if(indicatorProjectTemplateService.saveOrUpdate(template)){
            //保存 模板的详情
            if(CollUtil.isNotEmpty(dto.getTemplateDetails())){
                //先删除
                indicatorProjectTemplateDetailService.remove(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                        .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId,dto.getId()));
                analysisTemplateDetails(dto.getTemplateDetails(),template.getId());

                //校检 3个阶段 总分是不是100
                List<PerformanceIndicatorProjectTemplateDetail> stages = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                        .eq(PerformanceIndicatorProjectTemplateDetail::getType, PerformanceTemplateTypeEnum.STAGE.getCode())
                        .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, template.getId())
                        .in(PerformanceIndicatorProjectTemplateDetail::getName, Lists.newArrayList(PerformanceTemplateStageNameEnum.STAGE_1.getDesc(),
                                PerformanceTemplateStageNameEnum.STAGE_2.getDesc(), PerformanceTemplateStageNameEnum.STAGE_3.getDesc())));
                if(CollUtil.isNotEmpty(stages)){
                    int sum = stages.stream().filter(p -> Objects.nonNull(p.getScore())).mapToInt(PerformanceIndicatorProjectTemplateDetail::getScore).sum();
                    VUtils.isTrue(sum != 100).throwMessage("总分不是100 操作失败!");
                }
            }
            return "保存成功";
        }
        throw new BizException("保存失败!");
    }

    /**
     * 解析 项目考核指标
     * @param templateDetails
     * @return
     */
    public void analysisTemplateDetails(
            List<ProjectTemplateDetailDTO> templateDetails,Long templateId) {
        if(CollUtil.isEmpty(templateDetails)){
            return;
        }
        analysisTemplateDetailsInner(templateDetails,templateId,null,null);
    }

    public void analysisTemplateDetailsInner(List<ProjectTemplateDetailDTO> templateDetails,
                                              Long templateId,Long parentId,Long stageId) {
        if(CollUtil.isEmpty(templateDetails)){
            return;
        }
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        for(ProjectTemplateDetailDTO detail : templateDetails){
            //计算阶段的
            PerformanceIndicatorProjectTemplateDetail templateDetail = BeanUtil.copyProperties(detail, PerformanceIndicatorProjectTemplateDetail.class);
            templateDetail.setCreateBy(user.getUsername());
            templateDetail.setCreateOn(LocalDateTime.now());
            templateDetail.setUpdateBy(user.getUsername());
            templateDetail.setUpdateOn(LocalDateTime.now());
            templateDetail.setTemplateId(templateId);
            templateDetail.setParentId(parentId);
            //如果是阶段的计算下 总分
            if(!PerformanceTemplateTypeEnum.STAGE.getCode().equals(templateDetail.getType())){
                templateDetail.setStageId(stageId);
            }
            indicatorProjectTemplateDetailService.save(templateDetail);
            //如果是阶段的话 阶段Id 就是自己的id
            if(PerformanceTemplateTypeEnum.STAGE.getCode().equals(templateDetail.getType())){
                templateDetail.setScore(computeTotalScore(templateDetail));
                templateDetail.setStageId(templateDetail.getId());
                indicatorProjectTemplateDetailService.updateById(templateDetail);
                stageId = templateDetail.getId();
            }

            if(CollUtil.isNotEmpty(detail.getChildren())){
                analysisTemplateDetailsInner(detail.getChildren(),templateId,templateDetail.getId(),stageId);
            }
        }
    }

    /**
     * 计算 此阶段下的 总分
     * @param templateDetail
     * @return
     */
    private Integer computeTotalScore(PerformanceIndicatorProjectTemplateDetail templateDetail) {
        Integer total = 0;
        if(Objects.isNull(templateDetail) || Objects.isNull(templateDetail.getId())){
            return total;
        }

        if(PerformanceTemplateTypeEnum.STAGE.getCode().equals(templateDetail.getType())){
            List<PerformanceIndicatorProjectTemplateDetail> thirds = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                    .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, templateDetail.getTemplateId())
                    .eq(PerformanceIndicatorProjectTemplateDetail::getStageId, templateDetail.getId())
                    .eq(PerformanceIndicatorProjectTemplateDetail::getType,PerformanceTemplateTypeEnum.THIRD_INDEX.getCode()));
            if(CollUtil.isNotEmpty(thirds)){
                total = thirds.stream().mapToInt(PerformanceIndicatorProjectTemplateDetail::getScore).sum();
            }
        }
        return total;
    }

    /**
     * 装配 模板详情
     * @param templateDetails
     * @return
     */
    private List<ProjectTemplateDetailVO> buildTemplateDetail(List<PerformanceIndicatorProjectTemplateDetail> templateDetails) {
        List<ProjectTemplateDetailVO> res = Lists.newArrayList();
        //第一层是 一级的
        for(PerformanceIndicatorProjectTemplateDetail templateDetail : templateDetails){
            if(PerformanceTemplateTypeEnum.STAGE.getCode().equals(templateDetail.getType())){
                ProjectTemplateDetailVO detailVo = BeanUtil.copyProperties(templateDetail, ProjectTemplateDetailVO.class);
                List<ProjectTemplateDetailVO> secondList = Lists.newArrayList();
                for(PerformanceIndicatorProjectTemplateDetail templateDetailSecond : templateDetails) {
                    if (Objects.nonNull(templateDetailSecond.getParentId()) &&
                            templateDetailSecond.getParentId().equals(templateDetail.getId())) {
                        ProjectTemplateDetailVO detailSecondVo = BeanUtil.copyProperties(templateDetailSecond, ProjectTemplateDetailVO.class);
                        List<ProjectTemplateDetailVO> thirdList = Lists.newArrayList();
                        for(PerformanceIndicatorProjectTemplateDetail templateDetailThird : templateDetails) {
                            if (Objects.nonNull(templateDetailThird.getParentId()) &&
                                    templateDetailThird.getParentId().equals(templateDetailSecond.getId())) {
                                ProjectTemplateDetailVO detailThirdVo = BeanUtil.copyProperties(templateDetailThird, ProjectTemplateDetailVO.class);
                                List<ProjectTemplateDetailVO> fourList = Lists.newArrayList();
                                for(PerformanceIndicatorProjectTemplateDetail templateDetailFour : templateDetails) {
                                    if (Objects.nonNull(templateDetailFour.getParentId()) &&
                                            templateDetailFour.getParentId().equals(templateDetailThird.getId())) {
                                        ProjectTemplateDetailVO detailFourVo = BeanUtil.copyProperties(templateDetailFour, ProjectTemplateDetailVO.class);
                                        fourList.add(detailFourVo);
                                    }
                                }
                                detailThirdVo.setChildren(fourList);
                                thirdList.add(detailThirdVo);
                            }
                        }
                        detailSecondVo.setChildren(thirdList);
                        secondList.add(detailSecondVo);
                    }
                }
                detailVo.setChildren(secondList);
                res.add(detailVo);
            }
        }
        return res;
    }

    /**
     * 删除项目模板
     * @param id
     * @return
     */
    public String projectTemplateDelete(Long id) {
        PerformanceIndicatorProjectTemplate projectTemplate = indicatorProjectTemplateService.getById(id);
        VUtils.isTrue(Objects.isNull(projectTemplate)).throwMessage("该模板不存在!");

        if(CommonEnum.YES.getCode().equals(projectTemplate.getStatus())){
            throw new BizException("操作失败 该模板 是启用状态!");
        }

        if(indicatorProjectTemplateService.removeById(id)){
            indicatorProjectTemplateDetailService.remove(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                    .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId,id));
            return "删除成功";
        }
        return "操作失败";
    }
}
