package com.ningdatech.pmapi.performance.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisal;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalProject;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalProjectVO;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalVO;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalApplicationService;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalProjectService;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalService;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectApplicationService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Classname OrgSelfAppraisalManage
 * @Description
 * @Date 2023/6/19 14:18
 * @Author PoffyZhang
 */
@Component
@Slf4j
@AllArgsConstructor
public class OrgSelfAppraisalManage {

    private final IPerformanceAppraisalService performanceAppraisalService;

    private final IPerformanceAppraisalProjectService performanceAppraisalProjectService;

    private final IPerformanceAppraisalApplicationService performanceAppraisalApplicationService;

    private final IProjectService projectService;

    private final IProjectApplicationService applicationService;

    /**
     * 绩效列表
     * @param req
     * @return
     */
    public PageVo<PerformanceAppraisalVO> list(PerformanceAppraisalListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        //当前登录用户 单位code
        String empPosUnitCode = user.getEmpPosUnitCode();
        List<Project> projectLists = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getBuildOrgCode, empPosUnitCode));

        if(CollUtil.isEmpty(projectLists)){
            return PageVo.empty();
        }

        List<Long> projectIds = projectLists.stream().map(Project::getId).collect(Collectors.toList());

        List<PerformanceAppraisalProject> paps = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .in(PerformanceAppraisalProject::getProjectId, projectIds));
        if(CollUtil.isEmpty(paps)){
            return PageVo.empty();
        }

        Set<Long> paIds = paps.stream().map(PerformanceAppraisalProject::getAppraisalId).collect(Collectors.toSet());

        Page<PerformanceAppraisal> page = req.page();
        LambdaQueryWrapper<PerformanceAppraisal> wrapper = Wrappers.lambdaQuery(PerformanceAppraisal.class)
                        .in(PerformanceAppraisal::getId,paIds)
                        .orderByDesc(PerformanceAppraisal::getUpdateOn);
        performanceAppraisalService.page(page,wrapper);
        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<PerformanceAppraisalVO> res = page.getRecords().stream().map(p -> {
            PerformanceAppraisalVO vo = BeanUtil.copyProperties(p,PerformanceAppraisalVO.class);
            return vo;
        }).collect(Collectors.toList());
        return PageVo.of(res,page.getTotal());
    }

    public PageVo<PerformanceAppraisalProjectVO> pendingList(Long planId, PerformanceAppraisalListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        PerformanceAppraisal plan = performanceAppraisalService.getById(planId);
        VUtils.isTrue(Objects.isNull(plan)).throwMessage("该计划不存在!");

        List<PerformanceAppraisalProject> paps = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .eq(PerformanceAppraisalProject::getAppraisalId, plan.getId()));

        if(CollUtil.isEmpty(paps)){
            return PageVo.empty();
        }
        Set<String> projectIds = paps.stream().map(PerformanceAppraisalProject::getProjectCode).collect(Collectors.toSet());

        Page<Project> page = req.page();
        LambdaQueryWrapper<Project> wrapper = Wrappers.lambdaQuery(Project.class)
                .in(Project::getProjectCode, projectIds)
                .eq(Project::getNewest, Boolean.TRUE)
                .eq(Project::getBuildOrgCode, user.getEmpPosUnitCode())
                .like(StringUtils.isNotBlank(req.getProjectName()),Project::getProjectName,req.getProjectName());
        projectService.page(page,wrapper);

        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<PerformanceAppraisalProjectVO> res = page.getRecords().stream()
                .map(p -> {
                    PerformanceAppraisalProjectVO vo = BeanUtil.copyProperties(p, PerformanceAppraisalProjectVO.class);
                    vo.setCanSelfAppraisal(checkCanSelfAppraisal(plan));
                    return vo;
                })
                .collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    /**
     * 已经自评表
     * @param planId
     * @return
     */
    public PageVo<PerformanceAppraisalProjectVO> appraisaledList(Long planId) {

        return PageVo.empty();
    }

    /**
     * 是否可以自评
     * @param plan
     * @return
     */
    private Boolean checkCanSelfAppraisal(PerformanceAppraisal plan) {
        if(LocalDateTime.now().compareTo(plan.getSelfAppraisalStart()) > 0 &&
                LocalDateTime.now().compareTo(plan.getSelfAppraisalEnd()) < 0){
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }
}
