package com.ningdatech.pmapi.performance.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.util.BizUtils;
import com.ningdatech.pmapi.common.util.CodeUtil;
import com.ningdatech.pmapi.performance.convert.PerformanceAppraisalConveter;
import com.ningdatech.pmapi.performance.model.dto.AppraisalProjectDTO;
import com.ningdatech.pmapi.performance.model.dto.PerformanceAppraisalCreateDTO;
import com.ningdatech.pmapi.performance.model.dto.PerformanceAppraisalEditDTO;
import com.ningdatech.pmapi.performance.model.dto.PerformanceAppraisalProjectGroupSaveDTO;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisal;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalApplication;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalProject;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalProjectGroup;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalProjectGroupVO;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalProjectVO;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalVO;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalApplicationService;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalProjectGroupService;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalProjectService;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalService;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectApplication;
import com.ningdatech.pmapi.projectlib.service.IProjectApplicationService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Classname PerformanceAppraisalManage
 * @Description
 * @Date 2023/6/19 14:18
 * @Author PoffyZhang
 */
@Component
@Slf4j
@AllArgsConstructor
public class PerformanceAppraisalPlanManage {

    private final IPerformanceAppraisalService performanceAppraisalService;

    private final IPerformanceAppraisalProjectService performanceAppraisalProjectService;

    private final IPerformanceAppraisalApplicationService performanceAppraisalApplicationService;

    private final IProjectService projectService;

    private final IProjectApplicationService applicationService;

    private final IPerformanceAppraisalProjectGroupService groupService;

    private final UserInfoHelper userInfoHelper;

    /**
     * 绩效列表
     * @param req
     * @return
     */
    public PageVo<PerformanceAppraisalVO> list(PerformanceAppraisalListReq req) {
        Page<PerformanceAppraisal> page = req.page();
        LambdaQueryWrapper<PerformanceAppraisal> wrapper = Wrappers.lambdaQuery(PerformanceAppraisal.class)
                        .like(StringUtils.isNotBlank(req.getPlanName()),PerformanceAppraisal::getName,req.getPlanName())
                        .ge(Objects.nonNull(req.getStart()),PerformanceAppraisal::getCreateOn,req.getStart())
                        .le(Objects.nonNull(req.getEnd()),PerformanceAppraisal::getCreateOn,req.getEnd())
                        .orderByDesc(PerformanceAppraisal::getUpdateOn);
        performanceAppraisalService.page(page,wrapper);
        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<Long> paIds  = page.getRecords().stream().map(PerformanceAppraisal::getId).collect(Collectors.toList());
        List<PerformanceAppraisalProject> paps = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .in(PerformanceAppraisalProject::getAppraisalId, paIds));

        Map<String,Project> projectMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalProject>> papsGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paps)){
            papsGroupMap = paps.stream().collect(Collectors.groupingBy(PerformanceAppraisalProject::getAppraisalId));

            List<String> projectCodes = paps.stream().map(PerformanceAppraisalProject::getProjectCode).collect(Collectors.toList());
            List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                    .in(Project::getProjectCode, projectCodes)
                    .eq(Project::getNewest, Boolean.TRUE));
            if(CollUtil.isNotEmpty(projects)){
                projectMap = projects.stream().collect(Collectors.toMap(Project::getProjectCode,p -> p));
            }
        }

        List<PerformanceAppraisalApplication> paas = performanceAppraisalApplicationService.list(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .in(PerformanceAppraisalApplication::getAppraisalId, paIds));
        Map<Long,ProjectApplication> applicationMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalApplication>> paasGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paas)){
            paasGroupMap = paas.stream().collect(Collectors.groupingBy(PerformanceAppraisalApplication::getAppraisalId));

            List<Long> applicationIds = paas.stream().map(PerformanceAppraisalApplication::getApplicationId).collect(Collectors.toList());
            List<ProjectApplication> apps = applicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                    .in(ProjectApplication::getId, applicationIds));
            if(CollUtil.isNotEmpty(apps)){
                applicationMap = apps.stream().collect(Collectors.toMap(ProjectApplication::getId,a -> a));
            }
        }

        Map<Long, List<PerformanceAppraisalProject>> finalPapsGroupMap = papsGroupMap;
        Map<Long,List<PerformanceAppraisalApplication>> finalPaasGroupMap = paasGroupMap;
        Map<String,Project> finalProjectMap = projectMap;
        Map<Long,ProjectApplication> finalApplicationMap = applicationMap;
        List<PerformanceAppraisalVO> res = page.getRecords().stream().map(p -> {
            PerformanceAppraisalVO vo = BeanUtil.copyProperties(p,PerformanceAppraisalVO.class);
            if(finalPapsGroupMap.containsKey(p.getId())){
                vo.setProjects(PerformanceAppraisalConveter.convertProjects(finalPapsGroupMap.get(p.getId()),finalProjectMap));
            }
            if(finalPaasGroupMap.containsKey(p.getId())){
                vo.setApplications(PerformanceAppraisalConveter.convertApplications(finalPaasGroupMap.get(p.getId()),finalApplicationMap));
            }
            return vo;
        }).collect(Collectors.toList());
        return PageVo.of(res,page.getTotal());
    }

    public PerformanceAppraisalVO detail(Long id) {

        PerformanceAppraisal pa = performanceAppraisalService.getById(id);
        VUtils.isTrue(Objects.isNull(pa)).throwMessage("该计划不存在");

        PerformanceAppraisalVO vo = BeanUtil.copyProperties(pa,PerformanceAppraisalVO.class);
        //绩效关联的项目
        List<PerformanceAppraisalProject> paps = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .eq(PerformanceAppraisalProject::getAppraisalId, id));
        Map<String,Project> projectMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalProject>> papsGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paps)){
            papsGroupMap = paps.stream().collect(Collectors.groupingBy(PerformanceAppraisalProject::getAppraisalId));

            List<String> projectCodes = paps.stream().map(PerformanceAppraisalProject::getProjectCode).collect(Collectors.toList());
            List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                    .in(Project::getProjectCode, projectCodes)
                    .eq(Project::getNewest, Boolean.TRUE));
            if(CollUtil.isNotEmpty(projects)){
                projectMap = projects.stream().collect(Collectors.toMap(Project::getProjectCode,p -> p));
            }
        }

        //绩效关联的应用
        List<PerformanceAppraisalApplication> paas = performanceAppraisalApplicationService.list(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .eq(PerformanceAppraisalApplication::getAppraisalId, id));
        Map<Long,ProjectApplication> applicationMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalApplication>> paasGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paas)){
            paasGroupMap = paas.stream().collect(Collectors.groupingBy(PerformanceAppraisalApplication::getAppraisalId));

            List<Long> applicationIds = paas.stream().map(PerformanceAppraisalApplication::getApplicationId).collect(Collectors.toList());
            List<ProjectApplication> apps = applicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                    .in(ProjectApplication::getId, applicationIds));
            if(CollUtil.isNotEmpty(apps)){
                applicationMap = apps.stream().collect(Collectors.toMap(ProjectApplication::getId,a -> a));
            }
        }

        Map<Long, List<PerformanceAppraisalProject>> finalPapsGroupMap = papsGroupMap;
        Map<Long,List<PerformanceAppraisalApplication>> finalPaasGroupMap = paasGroupMap;
        Map<String,Project> finalProjectMap = projectMap;
        Map<Long,ProjectApplication> finalApplicationMap = applicationMap;
        if(finalPapsGroupMap.containsKey(vo.getId())){
            vo.setProjects(PerformanceAppraisalConveter.convertProjects(finalPapsGroupMap.get(vo.getId()),finalProjectMap));
        }
        if(finalPaasGroupMap.containsKey(vo.getId())){
            vo.setApplications(PerformanceAppraisalConveter.convertApplications(finalPaasGroupMap.get(vo.getId()),finalApplicationMap));
        }
        return vo;
    }

    /**
     * 新建计划
     * @param createDTO
     * @return
     */
    @Transactional
    public String create(PerformanceAppraisalCreateDTO createDTO) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        deduplicationName(createDTO);

        PerformanceAppraisal pa = BeanUtil.copyProperties(createDTO,PerformanceAppraisal.class);
        pa.setCreateOn(LocalDateTime.now());
        pa.setCreateBy(user.getUsername());
        pa.setUpdateOn(LocalDateTime.now());
        pa.setUpdateBy(user.getUsername());
        performanceAppraisalService.save(pa);

        if(CollUtil.isNotEmpty(createDTO.getProjects())
                && StringUtils.isNotBlank(createDTO.getTarget())
                && createDTO.getTarget().contains("1")){
            saveProjects(createDTO,pa.getId(),user);
        }

        if(CollUtil.isNotEmpty(createDTO.getApplicationIds())
                && StringUtils.isNotBlank(createDTO.getTarget())
                && createDTO.getTarget().contains("2")){
            saveApplications(createDTO,pa.getId(),user);
        }

        return "新建成功";
    }


    /**
     * 编辑计划
     * @param editDTO
     * @return
     */
    @Transactional
    public String edit(PerformanceAppraisalEditDTO editDTO) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        deduplicationName(editDTO);

        PerformanceAppraisal pa = BeanUtil.copyProperties(editDTO,PerformanceAppraisal.class);

        pa.setUpdateOn(LocalDateTime.now());
        pa.setUpdateBy(user.getUsername());
        performanceAppraisalService.updateById(pa);

        if(CollUtil.isNotEmpty(editDTO.getProjects())){
            saveProjects(editDTO,user);
        }

        if(CollUtil.isNotEmpty(editDTO.getApplicationIds())){
            saveApplications(editDTO,user);
        }

        return "编辑成功";
    }

    /**
     * 名称去重
     * @param dto
     */
    private void deduplicationName(PerformanceAppraisalCreateDTO dto) {
        long count = performanceAppraisalService.count(Wrappers.lambdaQuery(PerformanceAppraisal.class)
                .eq(PerformanceAppraisal::getName, dto.getName()));
        VUtils.isTrue(count > 0).throwMessage("名称已经存在");
    }

    private void deduplicationName(PerformanceAppraisalEditDTO dto) {
        if(StringUtils.isNotBlank(dto.getName())){
            long count =performanceAppraisalService.count(Wrappers.lambdaQuery(PerformanceAppraisal.class)
                    .eq(PerformanceAppraisal::getName,dto.getName())
                    .ne(Objects.nonNull(dto.getId()),PerformanceAppraisal::getId,dto.getId()));
            VUtils.isTrue(count > 0).throwMessage("名称已经存在");
        }
    }

    @Transactional
    public void saveProjects(PerformanceAppraisalCreateDTO createDTO,Long id,UserInfoDetails user) {
        if(CollUtil.isEmpty(createDTO.getProjects())){
            return;
        }

        List<AppraisalProjectDTO> projects = createDTO.getProjects();

        for(AppraisalProjectDTO appraisalProject : projects){
            Project newProject = projectService.getNewProject(appraisalProject.getId());
            VUtils.isTrue(Objects.isNull(newProject)).throwMessage("项目不存在 " + appraisalProject.getId());
            PerformanceAppraisalProject pap = new PerformanceAppraisalProject();
            pap.setAppraisalId(id);
            pap.setProjectId(appraisalProject.getId());
            pap.setProjectCode(newProject.getProjectCode());
            pap.setIsReAppraisal(appraisalProject.getIsReAppraisal());
            pap.setCreateBy(user.getUsername());
            pap.setCreateOn(LocalDateTime.now());
            pap.setUpdateBy(user.getUsername());
            pap.setUpdateOn(LocalDateTime.now());
            performanceAppraisalProjectService.save(pap);
        }
    }

    @Transactional
    public void saveApplications(PerformanceAppraisalCreateDTO createDTO,Long id,UserInfoDetails user) {
        if(CollUtil.isEmpty(createDTO.getApplicationIds())){
            return;
        }

        List<Long> applicationIds = createDTO.getApplicationIds();

        for(Long applicationId : applicationIds){
            ProjectApplication projectApplication = applicationService.getById(applicationId);
            VUtils.isTrue(Objects.isNull(projectApplication)).throwMessage("应用不存在 " + applicationId);
            PerformanceAppraisalApplication paa = new PerformanceAppraisalApplication();
            paa.setAppraisalId(id);
            paa.setApplicationId(applicationId);
            paa.setCreateBy(user.getUsername());
            paa.setCreateOn(LocalDateTime.now());
            paa.setUpdateBy(user.getUsername());
            paa.setUpdateOn(LocalDateTime.now());
            performanceAppraisalApplicationService.save(paa);
        }
    }

    @Transactional
    public void saveProjects(PerformanceAppraisalEditDTO editDTO,UserInfoDetails user) {
        if(Objects.isNull(editDTO.getProjects())){
            return;
        }

        //删除先
        performanceAppraisalProjectService.remove(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .eq(PerformanceAppraisalProject::getAppraisalId,editDTO.getId()));
        List<AppraisalProjectDTO> projects = editDTO.getProjects();
        for(AppraisalProjectDTO appraisalProject : projects){
            Project newProject = projectService.getNewProject(appraisalProject.getId());
            VUtils.isTrue(Objects.isNull(newProject)).throwMessage("项目不存在 " + appraisalProject.getId());
            PerformanceAppraisalProject pap = new PerformanceAppraisalProject();
            pap.setAppraisalId(editDTO.getId());
            pap.setProjectId(appraisalProject.getId());
            pap.setProjectCode(newProject.getProjectCode());
            pap.setIsReAppraisal(appraisalProject.getIsReAppraisal());
            pap.setCreateBy(user.getUsername());
            pap.setCreateOn(LocalDateTime.now());
            pap.setUpdateBy(user.getUsername());
            pap.setUpdateOn(LocalDateTime.now());
            performanceAppraisalProjectService.save(pap);
        }
    }

    @Transactional
    public void saveApplications(PerformanceAppraisalEditDTO editDTO,UserInfoDetails user) {
        if(Objects.isNull(editDTO.getApplicationIds())){
            return;
        }

        //删除先
        performanceAppraisalApplicationService.remove(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .eq(PerformanceAppraisalApplication::getAppraisalId,editDTO.getId()));
        List<Long> applicationIds = editDTO.getApplicationIds();

        for(Long applicationId : applicationIds){
            ProjectApplication projectApplication = applicationService.getById(applicationId);
            VUtils.isTrue(Objects.isNull(projectApplication)).throwMessage("应用不存在 " + applicationId);
            PerformanceAppraisalApplication paa = new PerformanceAppraisalApplication();
            paa.setAppraisalId(editDTO.getId());
            paa.setApplicationId(applicationId);
            paa.setCreateBy(user.getUsername());
            paa.setCreateOn(LocalDateTime.now());
            paa.setUpdateBy(user.getUsername());
            paa.setUpdateOn(LocalDateTime.now());
            performanceAppraisalApplicationService.save(paa);
        }
    }

    /**
     * 删除绩效评价计划
     * 连带删除 其下所有
     * @param id
     * @return
     */
    public String delete(Long id) {
        PerformanceAppraisal pa = performanceAppraisalService.getById(id);
        VUtils.isTrue(Objects.isNull(pa)).throwMessage("绩效评价不存在 删除失败!");

        //如果到了自评就不能删除了
        if(pa.getSelfAppraisalStart().compareTo(LocalDateTime.now()) < 0){
            throw new BizException("自评已经开始 不能够删除绩效评价");
        }

        if(performanceAppraisalService.removeById(id)){
            return "删除成功";
        }
        return "删除失败";
    }

    public List<PerformanceAppraisalProjectGroupVO> groupList(Long appraisalId,String name) {
        List<PerformanceAppraisalProjectGroup> groups = groupService.list(
                Wrappers.lambdaQuery(PerformanceAppraisalProjectGroup.class)
                .eq(PerformanceAppraisalProjectGroup::getAppraisalId, appraisalId)
                .like(StringUtils.isNotBlank(name),PerformanceAppraisalProjectGroup::getName,name));
        if(CollUtil.isEmpty(groups)){
            return Collections.emptyList();
        }

        Set<String> allEmployeeCodes = Sets.newHashSet();
        Set<String> projectCodes = Sets.newHashSet();
        Map<String, UserFullInfoDTO> userMap = Maps.newHashMap();
        Map<String, Project> projectMap = Maps.newHashMap();
        groups.stream().forEach(g -> {
            allEmployeeCodes.addAll(CodeUtil.convertStrToList(g.getReAppraisalUsers()));
            allEmployeeCodes.addAll(CodeUtil.convertStrToList(g.getVerificationUsers()));
            projectCodes.addAll(CodeUtil.convertStrToList(g.getProjectIds()));
        });

        if(CollUtil.isNotEmpty(allEmployeeCodes)){
            List<UserFullInfoDTO> users = userInfoHelper.getUserFullInfoByEmployeeCodes(allEmployeeCodes);
            if(CollUtil.isNotEmpty(users)){
                userMap = users.stream().collect(Collectors.toMap(UserFullInfoDTO::getEmployeeCode,u -> u));
            }
        }

        if(CollUtil.isNotEmpty(projectCodes)){
            List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                    .in(Project::getProjectCode,projectCodes)
                    .eq(Project::getNewest,Boolean.TRUE));
            if(CollUtil.isNotEmpty(projects)){
                projectMap = projects.stream().collect(Collectors.toMap(Project::getProjectCode,p -> p));
            }
        }

        Map<String, Project> finalProjectMap = projectMap;
        Map<String, UserFullInfoDTO> finalUserMap = userMap;
        return groups.stream().map(g -> {
            PerformanceAppraisalProjectGroupVO vo = new  PerformanceAppraisalProjectGroupVO();
            vo.setAppraisalId(g.getAppraisalId());
            vo.setName(g.getName());
            vo.setCreateOn(g.getCreateOn());
            vo.setCreateBy(g.getCreateBy());
            vo.setUpdateBy(g.getUpdateBy());
            vo.setUpdateOn(g.getUpdateOn());
            vo.setId(g.getId());
            vo.setProjects(checkProject(g.getProjectIds(), finalProjectMap));
            vo.setReAppraisalUsers(checkUser(g.getReAppraisalUsers(), finalUserMap));
            vo.setVerificationUsers(checkUser(g.getVerificationUsers(), finalUserMap));
            return vo;
        }).collect(Collectors.toList());
    }

    /**
     * 分组详情
     * @param id
     * @return
     */
    public PerformanceAppraisalProjectGroupVO groupDetail(Long id) {
        PerformanceAppraisalProjectGroup group = groupService.getById(id);

        if(Objects.isNull(group)){
            return null;
        }

        Map<String, Project> projectMap = Maps.newHashMap();
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .in(StringUtils.isNotBlank(group.getProjectIds()),Project::getProjectCode,
                        Arrays.stream(group.getProjectIds().split(StrPool.COMMA)).collect(Collectors.toList()))
                .eq(Project::getNewest,Boolean.TRUE));
        if(CollUtil.isNotEmpty(projects)){
            projectMap = projects.stream().collect(Collectors.toMap(Project::getProjectCode,p -> p));
        }

        Map<String, UserFullInfoDTO> userMap = Maps.newHashMap();
        Set<String> allEmployeeCodes = Sets.newHashSet();
        allEmployeeCodes.addAll(CodeUtil.convertStrToList(group.getReAppraisalUsers()));
        allEmployeeCodes.addAll(CodeUtil.convertStrToList(group.getVerificationUsers()));
        if(CollUtil.isNotEmpty(allEmployeeCodes)){
            List<UserFullInfoDTO> users = userInfoHelper.getUserFullInfoByEmployeeCodes(allEmployeeCodes);
            if(CollUtil.isNotEmpty(users)){
                userMap = users.stream().collect(Collectors.toMap(UserFullInfoDTO::getEmployeeCode,u -> u));
            }
        }

        PerformanceAppraisalProjectGroupVO vo = new PerformanceAppraisalProjectGroupVO();
        vo.setAppraisalId(group.getAppraisalId());
        vo.setName(group.getName());
        vo.setCreateOn(group.getCreateOn());
        vo.setCreateBy(group.getCreateBy());
        vo.setUpdateBy(group.getUpdateBy());
        vo.setUpdateOn(group.getUpdateOn());
        vo.setId(group.getId());
        vo.setProjects(checkProject(group.getProjectIds(), projectMap));
        vo.setReAppraisalUsers(checkUser(group.getReAppraisalUsers(), userMap));
        vo.setVerificationUsers(checkUser(group.getVerificationUsers(), userMap));
        return vo;
    }

    /**
     * 分组 保存
     * @param dto
     * @return
     */
    public String groupSave(PerformanceAppraisalProjectGroupSaveDTO dto) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        PerformanceAppraisalProjectGroup group = BeanUtil.copyProperties(dto, PerformanceAppraisalProjectGroup.class);

        if(Objects.isNull(dto.getId())){
            group.setId(null);
            group.setCreateOn(LocalDateTime.now());
            group.setCreateBy(user.getUsername());
        }else{
            PerformanceAppraisalProjectGroup old = groupService.getById(dto.getId());
            VUtils.isTrue(Objects.isNull(old)).throwMessage("该分组不存在 编辑失败！");
            group.setId(old.getId());
        }
        if(CollUtil.isNotEmpty(dto.getProjectCodes())){
            group.setProjectIds(dto.getProjectCodes().stream().collect(Collectors.joining(StrPool.COMMA)));
        }
        if(CollUtil.isNotEmpty(dto.getReAppraisalUsers())){
            group.setReAppraisalUsers(dto.getReAppraisalUsers().stream().collect(Collectors.joining(StrPool.COMMA)));
        }
        if(CollUtil.isNotEmpty(dto.getVerificationUsers())){
            group.setVerificationUsers(dto.getVerificationUsers().stream().collect(Collectors.joining(StrPool.COMMA)));
        }
        group.setUpdateOn(LocalDateTime.now());
        group.setUpdateBy(user.getUsername());
        groupService.saveOrUpdate(group);
        return "操作成功";
    }

    /**
     * 删除分组
     * @param id
     * @return
     */
    public String groupDelete(Long id) {
        PerformanceAppraisalProjectGroup group = groupService.getById(id);
        VUtils.isTrue(Objects.isNull(group)).throwMessage("分组不存在");

        //评价Id
        Long appraisalId = group.getAppraisalId();

        PerformanceAppraisal pa = performanceAppraisalService.getById(appraisalId);
        VUtils.isTrue(Objects.isNull(pa)).throwMessage("评价不存在");

        //如果到了自评就不能删除了
        if(pa.getSelfAppraisalStart().compareTo(LocalDateTime.now()) < 0){
            throw new BizException("自评已经开始 不能够删除绩效评价");
        }

        if(groupService.removeById(id)){
            return "删除成功";
        }
        return "删除失败";
    }

    private List<PerformanceAppraisalProjectVO> checkProject(String projectIds, Map<String, Project> projectMap) {
        if(StringUtils.isBlank(projectIds)){
            return Collections.emptyList();
        }
        return Arrays.stream(projectIds.split(StrPool.COMMA)).map(projectCode -> {
            PerformanceAppraisalProjectVO vo = new PerformanceAppraisalProjectVO();
            if(projectMap.containsKey(projectCode)){
                Project project = projectMap.get(projectCode);
                vo.setProjectTypeName(BizUtils.getProjectTypeName(project.getProjectType()));
                vo.setProjectType(project.getProjectType());
                vo.setId(project.getId());
                vo.setProjectCode(projectCode);
                vo.setProjectYear(project.getProjectYear());
                vo.setDeclareAmount(project.getDeclareAmount());
            }
            return vo;
        }).collect(Collectors.toList());
    }

    private List<UserFullInfoDTO> checkUser(String users, Map<String, UserFullInfoDTO> userMap) {
        if(StringUtils.isBlank(users)){
            return Collections.emptyList();
        }
        return Arrays.stream(users.split(StrPool.COMMA)).map(
           employeeCode -> userMap.get(employeeCode)
        ).collect(Collectors.toList());
    }
}
