package com.ningdatech.pmapi.performance.model.dto;

import com.alibaba.fastjson.annotation.JSONField;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalApplicationVO;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalProjectVO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @Classname PerformanceAppraisalCreateDTO
 * @Description 绩效评价对象
 * @Date 2023/6/19 14:02
 * @Author PoffyZhang
 */
@Data
@ApiModel(value = "绩效评价创建对象", description = "绩效评价创建对象")
public class PerformanceAppraisalCreateDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("计划名称")
    @NotBlank(message = "请输入计划名称")
    private String name;

    @ApiModelProperty("自评开始时间")
    @NotNull(message = "自评时间错误")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime selfAppraisalStart;

    @ApiModelProperty("自评结束时间")
    @NotNull(message = "自评时间错误")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime selfAppraisalEnd;

    @ApiModelProperty("复评开始时间")
    @NotNull(message = "复评时间错误")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime reAppraisalStart;

    @ApiModelProperty("复评结束时间")
    @NotNull(message = "复评时间错误")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime reAppraisalEnd;

    @ApiModelProperty("核查开始时间")
    @NotNull(message = "核查时间错误")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime verificationStart;

    @ApiModelProperty("核查结束时间")
    @NotNull(message = "核查时间错误")
    @JSONField(format = "yyyy-MM-dd")
    private LocalDateTime verificationEnd;

    @ApiModelProperty("评价目标 1项目 2应用 可多选 1,2")
    @NotBlank(message = "评价目标不能为空")
    private String target;

    @ApiModelProperty("评价项目 只有ID 和是否复评")
    private List<AppraisalProjectDTO> projects;

    @ApiModelProperty("评价应用ids")
    private List<Long> applicationIds;
}
