package com.ningdatech.pmapi.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.irs.manage.AppIrsManage;
import com.ningdatech.pmapi.irs.model.dto.ApiApplyDTO;
import com.ningdatech.pmapi.irs.model.dto.ApiApplySearchResult;
import com.ningdatech.pmapi.projectdeclared.model.entity.IrsApplication;
import com.ningdatech.pmapi.projectdeclared.model.entity.NdPiotTasks;
import com.ningdatech.pmapi.projectdeclared.model.vo.IrsApplicationVO;
import com.ningdatech.pmapi.projectdeclared.model.vo.PiotTasksVO;
import com.ningdatech.pmapi.projectdeclared.service.INdPiotTasksService;
import com.ningdatech.pmapi.projectdeclared.service.IrsApplicationService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.security.NoSuchAlgorithmException;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Classname IrsManage
 * @Description
 * @Date 2023/3/1 14:48
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class IrsManage {

    @Value("${irs.is-search-app}")
    private Boolean isSearchApp;

    private final IrsApplicationService irsApplicationService;

    private final INdPiotTasksService piotTasksService;

    private final AppIrsManage appIrsManage;


    public List<IrsApplicationVO> applicationList() {
        //不请求IRS
        if(!isSearchApp) {
            List<IrsApplication> list = irsApplicationService.list();
            if(CollUtil.isEmpty(list)){
                return Collections.emptyList();
            }
            return CollUtils.convert(list,app -> BeanUtil.copyProperties(app,IrsApplicationVO.class));
        }

        try {
            List<ApiApplySearchResult> apiApplySearchResults = appIrsManage.searchApps(new ApiApplyDTO());
            if(CollUtil.isNotEmpty(apiApplySearchResults)){
                return apiApplySearchResults.stream().map(apply -> {
                    IrsApplicationVO vo = new IrsApplicationVO();
                    vo.setApplicationCode(apply.getAppId());
                    vo.setApplicationName(apply.getName());
                    return vo;
                }).collect(Collectors.toList());
            }
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
        return Collections.emptyList();
    }

    public List<PiotTasksVO> listPiotTasks() {
        List<NdPiotTasks> list = piotTasksService.list();
        if(CollUtil.isEmpty(list)){
            return Collections.emptyList();
        }
        return CollUtils.convert(list,task -> BeanUtil.copyProperties(task,PiotTasksVO.class));
    }
}
